"""Fix more indices in compute manager log

Revision ID: f65aad4015cb
Revises: a27802e0d8fe
Create Date: 2023-06-25 10:33:38.594008

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "f65aad4015cb"
down_revision = "a27802e0d8fe"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("ix_compute_manager_log_timestamp", table_name="compute_manager_log")
    op.create_index("ix_compute_manager_log_manager_id", "compute_manager_log", ["manager_id"], unique=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("ix_compute_manager_log_manager_id", table_name="compute_manager_log")
    op.create_index("ix_compute_manager_log_timestamp", "compute_manager_log", ["timestamp"], unique=False)
    # ### end Alembic commands ###
