"""Change manager stats

Revision ID: 2c48247776d0
Revises: 84e421577a1a
Create Date: 2023-03-27 18:44:13.468201

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "2c48247776d0"
down_revision = "84e421577a1a"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("compute_manager", "total_worker_walltime")
    op.drop_column("compute_manager_log", "total_worker_walltime")

    op.alter_column("compute_manager", "total_task_walltime", new_column_name="total_cpu_hours")
    op.alter_column("compute_manager_log", "total_task_walltime", new_column_name="total_cpu_hours")
    # ### end Alembic commands ###


def downgrade():
    raise RuntimeError("Cannot downgrade")
