"""Remove versions table

Revision ID: 9a86ebfcfd1b
Revises: 5d1989f8b59f
Create Date: 2022-03-29 10:28:46.346932

"""

import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "9a86ebfcfd1b"
down_revision = "5d1989f8b59f"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("versions")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "versions",
        sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False),
        sa.Column("created_on", postgresql.TIMESTAMP(), autoincrement=False, nullable=True),
        sa.Column("elemental_version", sa.VARCHAR(), autoincrement=False, nullable=False),
        sa.Column("fractal_version", sa.VARCHAR(), autoincrement=False, nullable=False),
        sa.PrimaryKeyConstraint("id", name="versions_pkey"),
    )
    # ### end Alembic commands ###
