"""Add indices to timestamps in base_record

Revision ID: a2e52a5e90b6
Revises: b03545e6292a
Create Date: 2023-06-14 13:55:27.574071

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "a2e52a5e90b6"
down_revision = "b03545e6292a"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index("ix_base_record_created_on", "base_record", ["created_on"], unique=False, postgresql_using="brin")
    op.create_index("ix_base_record_modified_on", "base_record", ["modified_on"], unique=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("ix_base_record_modified_on", table_name="base_record")
    op.drop_index("ix_base_record_created_on", table_name="base_record", postgresql_using="brin")
    # ### end Alembic commands ###
