"""add response_bytes to access log

Revision ID: 79604526d271
Revises: 00f7a74dcc37
Create Date: 2021-05-05 09:15:03.601418

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "79604526d271"
down_revision = "00f7a74dcc37"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("access_log", sa.Column("response_bytes", sa.BigInteger(), nullable=True))
    op.add_column("access_log", sa.Column("request_bytes", sa.BigInteger(), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("access_log", "response_bytes")
    op.drop_column("access_log", "request_bytes")
    # ### end Alembic commands ###
