"""Add more indicies in association tables

Revision ID: 64cae90ab9fe
Revises: f65aad4015cb
Create Date: 2023-06-27 14:48:06.554515

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "64cae90ab9fe"
down_revision = "f65aad4015cb"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index(
        "ix_gridoptimization_optimization_id", "gridoptimization_optimization", ["optimization_id"], unique=False
    )
    op.create_index("ix_torsiondrive_optimization_id", "torsiondrive_optimization", ["optimization_id"], unique=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("ix_torsiondrive_optimization_id", table_name="torsiondrive_optimization")
    op.drop_index("ix_gridoptimization_optimization_id", table_name="gridoptimization_optimization")
    # ### end Alembic commands ###
