"""add indices to neb association tables

Revision ID: 363c647d7840
Revises: 2c48247776d0
Create Date: 2023-05-01 14:41:24.226258

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "363c647d7840"
down_revision = "2c48247776d0"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index("ix_neb_optimizations_optimization_id", "neb_optimizations", ["optimization_id"], unique=False)
    op.create_index("ix_neb_singlepoints_singlepoint_id", "neb_singlepoints", ["singlepoint_id"], unique=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("ix_neb_singlepoints_singlepoint_id", table_name="neb_singlepoints")
    op.drop_index("ix_neb_optimizations_optimization_id", table_name="neb_optimizations")
    # ### end Alembic commands ###
