"""Add more indices to results table

Revision ID: 9a6345275dba
Revises: 469ece903d76
Create Date: 2021-09-11 10:36:35.813038

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "9a6345275dba"
down_revision = "469ece903d76"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index("ix_results_basis", "result", ["basis"], unique=False)
    op.create_index("ix_results_driver", "result", ["driver"], unique=False)
    op.create_index("ix_results_keywords", "result", ["keywords"], unique=False)
    op.create_index("ix_results_method", "result", ["method"], unique=False)
    op.create_index("ix_results_molecule", "result", ["molecule"], unique=False)
    op.create_index("ix_results_program", "result", ["program"], unique=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("ix_results_program", table_name="result")
    op.drop_index("ix_results_molecule", table_name="result")
    op.drop_index("ix_results_method", table_name="result")
    op.drop_index("ix_results_keywords", table_name="result")
    op.drop_index("ix_results_driver", table_name="result")
    op.drop_index("ix_results_basis", table_name="result")
    # ### end Alembic commands ###
