"""add missing index on optimization_trajectory

Revision ID: 34872c09c1b1
Revises: e8bba86921a5
Create Date: 2022-04-08 14:22:18.851929

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "34872c09c1b1"
down_revision = "e8bba86921a5"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index(
        "ix_optimization_trajectory_singlepoint_id", "optimization_trajectory", ["singlepoint_id"], unique=False
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("ix_optimization_trajectory_singlepoint_id", table_name="optimization_trajectory")
    # ### end Alembic commands ###
