"""Add index to compute history

Revision ID: a27802e0d8fe
Revises: a2e52a5e90b6
Create Date: 2023-06-20 18:02:23.182750

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "a27802e0d8fe"
down_revision = "a2e52a5e90b6"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index("ix_record_compute_history_manager_name", "record_compute_history", ["manager_name"], unique=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("ix_record_compute_history_manager_name", table_name="record_compute_history")
    # ### end Alembic commands ###
