"""neb subtasks are transient

Revision ID: 377ef3aebfed
Revises: 4122a71f7031
Create Date: 2023-02-01 09:43:23.338150

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "377ef3aebfed"
down_revision = "4122a71f7031"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("neb_subtasks")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    raise RuntimeError("Cannot downgrade")
    # ### end Alembic commands ###
